#include "readplates.h"
#include "parsestring.h"
#include <fstream>
#include <stdlib.h>

using namespace parseString;

namespace licencePlates {

ostream& operator << (ostream &os, const plateReadError &err)
// Output operator for plate errors - print error and line no.
{
    os << err.getError();
    if (err.validLineNo()) {
    	os << " at line " << err.getLineNo();
    }
    return os;
}

plate::plate(string const & raw)
// Construct licence plate information from raw string
{
    stringTokeniser strtok(raw);
    if (strtok.getNoTokens() < 2)
    	throw plateReadError ("Unable to read plate line " + raw);
    reg= strtok.getElement(0);
    //cout << "Read plate " << reg << endl;
    if (strtok.getNoTokens() >= 3) {
    	//cout << "Comment starts at " << strtok.getPos(2) << endl;
    	comment= raw.substr(strtok.getPos(2));
	//cout << "Read comment " << comment << endl;
    } else {
    	comment="";
    }
    string timestr= strtok.getElement(1);
    try {
        hoursMins::basicTime tm(timestr);
    	myTime= tm;
    } catch (hoursMins::timeError e1) {
    	throw plateReadError (e1.getError());
    }
    // cout << "Read Time " << myTime.getTimeStr() << endl;   
 
}

ostream & operator << (ostream &os, const plate &thisPlate)
{
    string plateTime= thisPlate.getTimeStr();
    os << thisPlate.getReg() << " " << plateTime << " " <<
    	thisPlate.getComment();
    return os;
}


ostream & operator << (ostream &os, const plateList &plates)
{
    for (int i= 0; i < plates.size(); i++) {
    	os << plates.getElement(i) << endl;
    }
    return os;
}

bool operator == (const plate &left, const plate &right) 
{
    if (left.getReg() == right.getReg())
    	return true;
    return false;
}

plateList::plateList (string const &fname)
// Construct a list of plates from a file
{
    name= fname;
    ifstream readFile(fname.c_str());
    if (!readFile) {
        throw plateReadError ("Unable to open file "+fname);
    }
    string readLine;
    bool infoSet= false;
    int lineNo= 1;
    while (getline(readFile,readLine)) {
    	//cout << lineNo << endl;
	//cout << typeOfLine(readLine) << endl;
    	switch (typeOfLine(readLine)) {
	    case (plateList::lineTypeInfo) :
	    	if (infoSet == true) {
		    tidyup();
		    throw plateReadError("Second #INFO line found in file "+
		    	fname, lineNo);
		}
		infoSet= true;
		noVehicles= parseInfoLine(readLine);
		if (noVehicles <= 0) {
		    tidyup();
		    throw plateReadError("Incorrect #INFO line found in file "+
		   	fname, lineNo); 
		}
	    	break;
	    case (plateList::lineTypeComment) :
	    	// Ignore comments
		break;
	    case (plateList::lineTypeData): 
	    	try { 	
		    plate newplate(readLine); 
		    list.push_back (newplate);
		} catch (plateReadError e1){
		    tidyup();
		    throw plateReadError (e1.getError() + " in file " 
		    	+fname,lineNo);
		}
	    	break;
	    default : 
	    	cout << "Error" << endl;
	    	tidyup();
		throw plateReadError("Unrecognised line in file "+fname,
		    lineNo);
	    
	}
	lineNo++;
    }
    if (infoSet == false) {
    	tidyup();
	throw plateReadError ("No #INFO line in file "+fname);
    }  	
}

void plateList::tidyup()
// Clear anything necessary after constructor fails
{
    clearList();
}

int plateList::parseInfoLine(string input)
// Given that we have an input line, return the no of vehicles.
{
    string::size_type i1= input.find_first_of("0123456789");
    if (i1 == string::npos) // Return 0 if there are no digits
    	return 0;
    string num= input.substr(i1);
    int infonum= atoi(num.c_str());
    return infonum;
}

int plateList::typeOfLine(string input)
// Returns lineTypeInfo, lineTypeComment or lineTypeData
{
    if (input.size() == 0)
    	return plateList::lineTypeComment;
    if ("#INFO" == input.substr(0,5))
    	return plateList::lineTypeInfo;
    if (input[0] == '#')
    	return plateList::lineTypeComment;
    return plateList::lineTypeData;
}




} // End of namespace licencePlates
